<?php

/**
 *  Check it not called directly
 */
if( !defined('ABSPATH') ){
	echo "Hey there, direct access not allowed!";
	exit();
}

use OneLogin\Saml2\Auth;
use OneLogin\Saml2\Settings;


/**
 * Add Login with PureAuth button to footer
 * Page: Login Page
 * action: login_footer
 */
function pureauth_custom_login_button(){
	$pureauth_login_button_message = "Login with PureAUTH";
	?>
    <a href="?pureauth-saml" id="pureauth-login-button">
        <div class="pureauth-login-div">
            <img class="pureauth-login-logo" src="<?php echo esc_url(plugins_url('assets/icons/pureid_logo.png', dirname(__FILE__))); ?>" alt="">
            <p class="pureauth-login-text"><?php echo esc_html_e( $pureauth_login_button_message, 'pureauth-saml' ); ?></p>
        </div>
    </a>
	<?php
}

/**
 * Check PureAuth SAML support enabled or not
 */
function pureauth_saml_is_enabled()
{
	$pureauth_saml_enabled = get_option( "pureauth_saml_is_enabled", "false" );

	if( $pureauth_saml_enabled == "1" )
	{
		return true;
	}
	else
	{
		return false;
	}
}


/**
 * Initialize SAML
 */
function pureauth_initialize_saml()
{
	// Include required libraries
	require_once plugin_dir_path(__FILE__)."_toolkit_loader.php";
	require_once plugin_dir_path(__FILE__)."settings.php";

	if( !pureauth_saml_is_enabled() )
	{
		return false;
	}

	try{
		$auth = new Auth($settings);
	} catch( \Exception $e ) {
		echo "<br/>".__("PureAuth SAML plugin is not properly configured.", "pureauth-saml")."<br/>";
		echo esc_html($e->getMessage());
		exit();
	}

	return $auth;

}


/**
 * PureAuth SAML SSO Endpoint
 * It will send AuthNRequest to IdP
 */
function pureauth_saml_authentication()
{
	// Check user already logged in or not
	if( is_user_logged_in() )
	{
		return true;
	}

	$auth = pureauth_initialize_saml();

	if( $auth == false )
	{
		// Redirect to home page
		wp_redirect( home_url() );
		exit();
	}
	else{
		// Sends AuthNRequest
		$auth->login();
	}
	exit();
}


/**
 * PureAuth SAML ACS Endpoint
 * It will process SAMLResponse
 */
function pureauth_saml_acs()
{
	$auth = pureauth_initialize_saml();

	if( $auth == false )
	{
		// Redirect to home page
		wp_redirect( home_url() );
		exit();
	}

	$auth->processResponse();

	$errors = $auth->getErrors();

	if (!empty($errors)) {
		echo '<br>'.__("There was a error processing the SAML Response").': ';
		foreach($errors as $error) {
			echo esc_html($error).'<br>';
		}
		echo __("Contact the administrator");
		exit();
	}


	$nameid = $auth->getNameId();
	if( empty($nameid) )
	{
//		echo __("NameID attribute not found in SAMLResponse");
//		exit();
		add_filter( 'login_errors', 'pureauth_nameid_not_found_error' );
	}
	$email = sanitize_email( $nameid );

	if( !is_email( $email ) )
	{
//		echo __( "Provided email id is not valid!" );
//		exit();
		add_filter( 'login_errors', 'pureauth_email_not_valid_error' );
	}
    elseif( empty( $email ) )
	{
//		echo __( "Email not retrieved from SAMLResponse and EmailID is required." );
//		exit();
	    add_filter( 'login_errors', 'pureauth_email_not_received_error' );
	}


	// Check emailId exists or not

	$user_id = email_exists( $email );

	if($user_id)
	{
        // Check allowed roles
        $is_allowed = pureauth_check_allowed_roles( $user_id );

        if( $is_allowed )
        {
            // Authorise User
            wp_set_current_user( $user_id );
            wp_set_auth_cookie( $user_id );
            //Fire wp_login action to update login timestamp
	        $user = get_user_by( 'id', $user_id );
	        do_action( 'wp_login', $user->user_login, $user );
            wp_redirect(home_url());
        }
        else
        {
            add_filter( 'login_errors', 'pureauth_user_role_not_allowed_error' );
        }
	}
	else{
//		echo __("User not exists!");
//		exit();
		add_filter( 'login_errors', 'pureauth_user_not_exists_error' );
	}

	// User Authorization will go here

}

/**
 * PureAuth SAML Metadata Publisher
 * It will publish SAML metadata
 */
function pureauth_saml_metadata()
{
	require_once "_toolkit_loader.php";
	require_once "settings.php";

	$settingsInfo = new Settings( $settings, true );
	$metadata = $settingsInfo->getSPMetadata();

	header( "Content-Type: text/xml" );
	echo ent2ncr($metadata);
	exit();


}

/**
 * PureAuth SAML router
 */
function pureauth_saml_router()
{
	if( isset( $_GET["pureauth-saml-acs"] ) )
	{
		if( empty( $_POST['SAMLResponse'] ) )
		{
			echo "The ACS endpoint excepts a SAMLResponse value sent using HTTP-POST binding. No Post Response found.";
			exit();
		}
		pureauth_saml_acs();
	}
    elseif( isset( $_GET["pureauth-saml-metadata"] ) )
	{
		pureauth_saml_metadata();
	}
	// SAML SSO
    elseif( isset( $_GET["pureauth-saml"] ) )
	{
		pureauth_saml_authentication();
	}
}

/**
 * NameID not found Error
 * @return string
 */
function pureauth_nameid_not_found_error() {
	return '<strong>Error</strong>: NameID attribute not found in SAMLResponse.';
}

/**
 * Email ID not found Error
 * @return string
 */
function pureauth_email_not_valid_error() {
	return '<strong>Error</strong>: Provided Email ID is not valid!';
}

/**
 * Email ID not received Error
 * @return string
 */
function pureauth_email_not_received_error() {
	return '<strong>Error</strong>: Email not retrieved from SAMLResponse and Email ID is required.';
}

/**
 * User Not Exists Error
 * @return string
 */
function pureauth_user_not_exists_error($error) {
	return '<strong>Error</strong>: User does not exists.';
}

/**
 * User Role not allowed
 * @return string
 */
function pureauth_user_role_not_allowed_error( $error )
{
    return '<strong>Error</strong>: User role not allowed to use PureAuth Authentication.';
}


// Helper functions
/**
 * It will check user roles 
 * @param $user_id
 * @return boolean
 */
function pureauth_check_allowed_roles( $user_id )
{
	$user = new WP_User( $user_id );

	$user_roles = $user->roles;
	$pureauth_options_allowed_roles = get_option( 'pureauth_options_allowed_roles' );

	if( $pureauth_options_allowed_roles == false ) {
		$pureauth_options_allowed_roles = array();
    }

	foreach( $user_roles as $role )
	{
		if( in_array( $role, $pureauth_options_allowed_roles ) )
		{
			return true;
		}
	}
}